﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace CUEImporter
{
    public class BASS
    {
        /*
         * プラグイン構造体
         */
        private struct BASSPlugin
        {
            public IntPtr ptr;
            public string filename;
        }
        public const uint BASS_DEVICE_NOSPEAKER = 4096;
        private const uint BASS_UNICODE = 0x80000000;
        private static List<BASSPlugin> plugins = new List<BASSPlugin>();
        static bool available = false;

        /*
         * static constructor
         * 成功したらavailableをtrueに
         */
        static BASS(){
            try
            {
                BASS.available = BASS.BASS_Init(-1, 44100, BASS.BASS_DEVICE_NOSPEAKER, (IntPtr)0, (IntPtr)0);
            }
            catch
            {
            }
        }

        public static bool isAvailable()
        {
            return BASS.available;
        }
        /*
         * ファイル名で指定したplug-inを読み込む。
         */
        public static bool BASS_PluginLoad(string filename, uint flags)
        {
            IntPtr pinPtr = (IntPtr)0;
            try
            {
                pinPtr = _BASS_PluginLoad(filename, BASS_UNICODE | flags);
                if (pinPtr != (IntPtr)0)
                {
                    plugins.Add(new BASSPlugin { filename = filename, ptr = pinPtr });
                    return true;
                }
                else
                {
                    return false;
                }
            }
            catch (Exception)
            {
                return false;
            }
        }

        public static List<string> getLoadedPlugins()
        {
            List<string> list = new List<string>();
            foreach (BASSPlugin p in plugins)
            {
                list.Add(p.filename);
            }
            return list;
        }

        public static IntPtr BASS_StreamCreateFile(Boolean ismemory, string filename, uint offset, uint length, uint flags)
        {
            try
            {
                return _BASS_StreamCreateFile(ismemory, filename, offset, length, BASS_UNICODE | flags);
            }
            catch (Exception)
            {
                return (IntPtr)0;
            }
        }

        public static bool BASS_StreamFree(IntPtr stream)
        {
            bool ret = false;
            try
            {
                ret = _BASS_StreamFree(stream);
            }
            catch (Exception)
            {
                ret = false;
            }
            return ret;
        }

        public static void Dispose()
        {
            foreach (BASSPlugin plugin in plugins)
            {
                try
                {
                    _BASS_PluginFree(plugin.ptr);
                }
                catch { }
            }
            plugins.Clear();
            try
            {
                _BASS_Free();
            }
            catch
            {
            }
        }
        [DllImport("bass.dll", EntryPoint = "BASS_Init")]
        private static extern Boolean BASS_Init(int device, uint freq, uint flags, IntPtr hwnd, IntPtr guid);

        [DllImport("bass.dll", EntryPoint = "BASS_PluginLoad", CharSet = CharSet.Unicode)]
        private static extern IntPtr _BASS_PluginLoad(string filename, uint flags);

        [DllImport("bass.dll", EntryPoint = "BASS_PluginFree", CharSet = CharSet.Unicode)]
        private static extern bool _BASS_PluginFree(IntPtr plugin);

        [DllImport("bass.dll", EntryPoint = "BASS_StreamCreateFile", CharSet = CharSet.Unicode)]
        private static extern IntPtr _BASS_StreamCreateFile(Boolean ismemory, string filename, UInt64 offset, UInt64 length, uint flags);

        [DllImport("bass.dll", EntryPoint = "BASS_StreamFree")]
        private static extern bool _BASS_StreamFree(IntPtr stream);

        [DllImport("bass.dll", EntryPoint = "BASS_ChannelGetLength")]
        public static extern UInt64 BASS_ChannelGetLength(IntPtr handle, uint mode);

        [DllImport("bass.dll", EntryPoint = "BASS_ChannelBytes2Seconds")]
        public static extern float BASS_ChannelBytes2Seconds(IntPtr handle, UInt64 pos);

        [DllImport("bass.dll", EntryPoint = "BASS_ErrorGetCode")]
        public static extern int BASS_ErrorGetCode();

        [DllImport("bass.dll", EntryPoint = "BASS_Free")]
        private static extern bool _BASS_Free();
    }
}

