﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;

namespace CUEImporter
{
    class CD
    {
        private static System.DateTime UnixEpoch = new System.DateTime(1970, 1, 1, 0, 0, 0);
        public enum H2k6Codec { NA, MP1, MP2, MP3, OGG, WAV, WMA, AAC, MP4, ALAC };
        private const string insertFormat = "INSERT INTO list VALUES('{0}','{1}','{2}',{3},'{4}','{5}','{6}','{7}','{8}','{9}',{10},'','',{11},2,44100,{12},1,{13},'{14}+CUE',32,0,0,0,0,{15});";
        private const string updateFormat = "UPDATE list SET 'file_title'='{1}','file_ext'='{2}','file_size'={3},'tagTitle'='{4}','tagArtist'='{5}','tagAlbum'='{6}','tagGenre'='{7}','tagDate'='{8}','tagComment'='{9}','tagTracknumber'={10},'statDuration'={11},'statBitrate'={12},'infoCodec'={13},'infoCodec_sub'='{14}+CUE','modify'={15} WHERE file_name = '{0}';";

        public string filename;
        public string artist;
        public string album;
        public string date;
        public TimeSpan length;
        public float flength;
        public string genre;
        public string comment;
        public List<Track> tracks = null;

        public CD()
        {
            filename = "";
            tracks = new List<Track>();
            artist = "";
            album = "";
            date = "";
            length = new TimeSpan(0);
            flength = 0;
            genre = "";
            comment = "";
        }
        public CD(string filename)
        {
        }
        private string escapeSinglequot(string str)
        {
            if (str == null) return "";
            return (new Regex("'")).Replace(str, "''");
        }
        private string toSQLString(string format){
            StringBuilder buf = new StringBuilder();
            for (int i = 0, l = this.tracks.Count; i < l; i++) {
                if (this.tracks[i].filename.Equals("")) continue;
                this.tracks[i].SQLInit();
                string line = String.Format(format
                    , escapeSinglequot(this.filename + new String(' ', (i + 1))) // cueファイル名+スペース*tr
                    , escapeSinglequot(this.tracks[i].filename_body) //this.tracks[tr].filename//.Split(".").Last<string> // 実体ファイル名
                    , escapeSinglequot("." + this.tracks[i].filename_ext.ToLower())
                    , this.tracks[i].filesize.ToString() // filesize
                    , escapeSinglequot(this.tracks[i].title)
                    , escapeSinglequot((this.tracks[i].artist == null ? this.artist : this.tracks[i].artist))
                    , escapeSinglequot(this.album)
                    , escapeSinglequot(this.genre)
                    , escapeSinglequot(this.date)
                    , escapeSinglequot((this.tracks[i].comment == null ? this.comment : this.tracks[i].comment))
                    , (i + 1).ToString()
                    , this.tracks[i].duration // duration
                    , this.tracks[i].bitrate
                    , (int)this.tracks[i].codec
                    , escapeSinglequot(this.tracks[i].filename_ext.ToUpper())
                    , (int)((System.DateTime.Now.ToFileTime() - UnixEpoch.ToFileTime()) / 10000000)
                );
                buf.AppendLine(line);
            }
            return buf.ToString();
        }
        public string ToUPDATEString()
        {
            return toSQLString(updateFormat);
        }
        public string ToINSERTString()
        {
            return toSQLString(insertFormat);
        }
        public class Track
        {
            public string artist = null;
            public string title = null;
            public string comment = null;
            public string filename = null;
            public int bitrate = 1410;
            public int start = 0;
            public int end = 100 * 60;

            public H2k6Codec codec = H2k6Codec.NA;
            public string filename_body = null;
            public string filename_ext = null;
            public int duration = 0;
            public int filesize = 0;
            public void SQLInit()
            {
                filename_body = (new Regex(@"\.[^\.]+$")).Replace(this.filename, "");
                filename_ext = (new Regex(@"\.(?<1>[^\.]+)$")).Match(this.filename).Groups[1].Value;
                switch (filename_ext.ToUpper()) {
                    case "MP3":
                        codec = H2k6Codec.MP3;
                        break;
                    case "AAC":
                        codec = H2k6Codec.AAC;
                        break;
                    case "M4A":
                    case "MP4":
                        if (this.bitrate > 350) { // たぶん320だけど、若干余裕もたす
                            codec = H2k6Codec.ALAC;
                        } else {
                            codec = H2k6Codec.AAC; // このへんは微妙なのでAACにしてしまう
                        }
                        break;
                    case "OGG":
                        codec = H2k6Codec.OGG;
                        break;
                    case "WAV":
                        codec = H2k6Codec.WAV;
                        break;
                    case "WMA":
                    case "ASF":
                        codec = H2k6Codec.WMA;
                        break;
                    case "FLAC":
                    case "FLA":
                        codec = H2k6Codec.WAV;
                        filename_ext = "FLAC";
                        break;
                    case "APE":
                    case "WV":
                    case "TTA":
                    case "TAK":
                        codec = H2k6Codec.WAV;
                        break;
                }
                duration = (int)(this.end > this.start ? ((this.end - this.start) / 75) : 0);
                filesize = bitrate / 8 * duration;
            }
        }
    }
    class CUEparse
    {
        private static Regex[] res = 
            {
                new Regex(@"(?<KEY>TRACK) (?<1>.*) AUDIO"),
                new Regex(@"(?<KEY>INDEX 01) (?<1>.*):(?<2>.*):(?<3>.*)"),
                new Regex(@"(?<KEY>TITLE) ""(?<1>.*)"""),
                new Regex(@"(?<KEY>TITLE) (?<1>.*)"),
                new Regex(@"(?<KEY>PERFORMER) ""(?<1>.*)"""),
                new Regex(@"(?<KEY>PERFORMER) (?<1>.*)"),
                new Regex(@"(?<KEY>FILE) ""(?<1>.*)"""),
                new Regex(@"(?<KEY>FILE) (?<1>.*)"),
                new Regex(@"(?<KEY>REM COMMENT) ""(?<1>.*)"""),
                new Regex(@"(?<KEY>REM COMMENT) (?<1>.*)"),
                new Regex(@"(?<KEY>REM GENRE) ""(?<1>.*)"""),
                new Regex(@"(?<KEY>REM GENRE) (?<1>.*)"),
                new Regex(@"(?<KEY>REM DATE) ""(?<1>.*)"""),
                new Regex(@"(?<KEY>REM DATE) (?<1>.*)"),
            };

        public static CD openFile(string filename){ // このめそっどきもい
            CD cd = new CD();
            bool isGlobal = true;
            string last_filename = "";
            int last_length = 0;
            int last_bitrate = 0; // bits per sec
            Match m;

            cd.filename = filename;
            string[] lines = File.ReadAllLines(filename,Encoding.Default);
            int lastIndex = 0; //末尾インデックスのキャッシュ
            for (int i = 0; i < lines.Length; i++)
            {
                string line = lines[i];
                for (int j = 0; j < res.Length; j++)
                {
                    m = res[j].Match(line);
                    if (m.Success)
                    {
                        switch(m.Groups["KEY"].Value){
                            case "FILE": // FILEのとき
                                string realfile;
                                last_filename = m.Groups[1].Value;
                                if (Path.IsPathRooted(last_filename))
                                {
                                    realfile = last_filename;
                                }
                                else
                                {
                                    realfile = Path.GetDirectoryName(filename) + Path.DirectorySeparatorChar + last_filename;
                                }
                                if (File.Exists(realfile)) {
                                    float sec = AudioLengthGetter.instance.getLength(realfile);
                                    cd.length = cd.length.Add(new TimeSpan(0,0,0,(int)sec,(int)(sec*1000)%1000));
                                    last_length = (int)(sec * 75);
                                    if (sec > 0) {
                                        last_bitrate = (int)((new FileInfo(realfile)).Length * 8 / sec);
                                    } else {
                                        last_bitrate = 0;
                                    }
                                } else {
                                    last_filename = "";
                                    last_bitrate = 1410;
                                }
                                break;
                            case "PERFORMER": // PERFORMERのとき
                                if (isGlobal) {
                                    cd.artist = m.Groups[1].Value;
                                } else {
                                    cd.tracks[lastIndex].artist = m.Groups[1].Value;
                                }
                                break;
                            case "TITLE": // TITLEのとき
                                if (isGlobal) {
                                    cd.album = m.Groups[1].Value;
                                } else {
                                    cd.tracks[lastIndex].title = m.Groups[1].Value;
                                }
                                break;
                            case "REM GENRE": // REM GENREのとき
                                cd.genre = m.Groups[1].Value;
                                break;
                            case "REM DATE": // REM DATEのとき
                                cd.date = m.Groups[1].Value;
                                break;
                            case "REM COMMENT": // REM COMMENTのとき
                                if (isGlobal) {
                                    cd.comment = m.Groups[1].Value;
                                } else {
                                    cd.tracks[lastIndex].comment = m.Groups[1].Value;
                                }
                                break;
                            case "TRACK": // TRACK nn AUDIO のとき
                                cd.tracks.Add(new CD.Track());
                                lastIndex = cd.tracks.Count - 1;
                                cd.tracks[lastIndex].filename = last_filename;
                                cd.tracks[lastIndex].bitrate = last_bitrate;
                                cd.tracks[lastIndex].end = last_length;
                                isGlobal = false;
                                break;
                            case "INDEX 01": // INDEX 01 00:00:00のとき
                                cd.tracks[lastIndex].start = (Convert.ToInt32(m.Groups[1].Value)*60+Convert.ToInt32(m.Groups[2].Value))*75+Convert.ToInt32(m.Groups[3].Value);
                                if (lastIndex > 0) {
                                    if (cd.tracks[lastIndex - 1].filename == last_filename) {
                                        cd.tracks[lastIndex - 1].end = cd.tracks[lastIndex].start;
                                    }
                                }
                                break;
                        }
                        break;
                    }
                }
            }
            // ミリ秒をカット
            cd.length = cd.length.Subtract(new TimeSpan(0,0,0,0,cd.length.Milliseconds));

            return cd;
        }
    }
}
