﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace CUEImporter
{
    public partial class Form1 : Form
    {
        private delegate void dTrace(traceType type, string str);
        private volatile Queue<CD> SQLQue;
        SQLite3DB libraryDB;
        Thread importThread = null;

        private void ProgressStep(ProgressBar bar)
        {
            lock (bar)
            {
                bar.PerformStep();
            }
        }
        private void ProgressFin(ProgressBar bar)
        {
            bar.Value = 0;
            bar.Maximum = 0;
        }
        private void ProgressAddMax(ProgressBar bar, int num)
        {
            lock (bar)
            {
                bar.Maximum += num;
            }
        }
        public delegate void DPVoid(ProgressBar bar);
        public delegate void DPInt(ProgressBar bar,int i);
        private enum traceType
        {
            INFO, ERROR
        }
        private void trace(traceType type,string str)
        {
            if (myconsole.InvokeRequired) {
                this.Invoke(new dTrace(trace), new object[] { type,str });
            } else {
                myconsole.AppendText("["+type.ToString()+"] " + str);
            }
        }
        public Form1()
        {
            InitializeComponent();
        }
        private delegate void dSetTitle(string title);
        private void setTitle(string str)
        {
            if (this.InvokeRequired) {
                this.Invoke(new dSetTitle(setTitle), new object[] { str });
            } else {
                this.Text="CUEシート インポータ "+str;
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            SQLQue = new Queue<CD>();
            trace(traceType.INFO,AudioLengthGetter.instance.getStatusMsg());
        }

        private void loadProc(object _filename)
        {
            string filename = (string)_filename;
            try
            {
                setTitle("処理中... " + progressBarLoad.Value +" / "+ progressBarLoad.Maximum);
                processCUE(filename);
            }
            catch (Exception ex)
            {
                trace(traceType.ERROR,ex.Message);
            }
            finally
            {
                this.Invoke(new DPVoid(ProgressStep), new object[] { progressBarLoad });
                if (progressBarLoad.Maximum == progressBarLoad.Value)
                {
                    setTitle("");
                }

            }
        }
        private void runImport() // import thread
        {
            int inserted = 0;
            int updated = 0;
            int c = 0;
            try {
                libraryDB.Exec("BEGIN;");
                trace(traceType.INFO,"library.dbへのインポートを開始しました\n");
            } catch {
                trace(traceType.ERROR, "library.dbへのインポートを開始できませんでした\n");
                return;
            }
            c = SQLQue.Count;
            CD currentCD;
            string currentSQL;
            object[] _progressBarInsert = { progressBarInsert };
            this.Invoke(new DPInt(ProgressAddMax), new object[] { progressBarInsert, SQLQue.Count });
            while (SQLQue.Count > 0) {
                lock(SQLQue){
                    currentCD = SQLQue.Dequeue();
                }
                currentSQL = currentCD.ToINSERTString();
                this.Invoke(new DPVoid(ProgressStep), _progressBarInsert);
                try {
                    libraryDB.Exec(currentSQL);
                    inserted++;
                } catch (SQLite3DB.SQLite3Exception) {
                    try {
                        libraryDB.Exec(currentCD.ToUPDATEString());
                        updated++;
                    } catch (SQLite3DB.SQLite3Exception) {
                    }
                }
            }
            try {
                libraryDB.Exec("COMMIT;");
                this.Invoke(new DPVoid(ProgressFin), _progressBarInsert);
                this.Invoke(new DPVoid(ProgressFin), new object[]{progressBarLoad});
                trace(traceType.INFO, String.Format("library.dbへのインポートが完了しました {0}/{1}ファイル成功 (INSERT={2}, UPDATE={3})\n", c, (inserted + updated), inserted, updated));
            } catch {
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            OpenFileDialog fileDLG = new OpenFileDialog();
            fileDLG.Filter = "CUEシート|*.cue";
            fileDLG.DefaultExt = "CUE";
            fileDLG.AddExtension = true;
            fileDLG.Multiselect = true;
            if(fileDLG.ShowDialog() == DialogResult.OK){
                enqueFiles(fileDLG.FileNames);
            }
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            if(libraryDB != null)libraryDB.Close();
        }

        private void Form1_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop)) {
                ThreadPool.QueueUserWorkItem(enqueFiles,e.Data.GetData(DataFormats.FileDrop));
            }

        }

        private void enqueFiles(object o){
            string[] pathlist = (string[])o;
            if (pathlist.Length > 0)
            {
                foreach (string filename in pathlist)
                {
                    if(System.IO.Directory.Exists(filename)){
                        trace(traceType.INFO, filename + "内を検索中です\n");
                        setTitle(filename + "内を検索中です");
                        string[] cuefiles = System.IO.Directory.GetFiles(filename, "*.cue", System.IO.SearchOption.AllDirectories);
                        trace(traceType.INFO, filename + "内にcueファイルを"+cuefiles.Length+"個発見しました\n");
                        enqueFiles(cuefiles);
                    }
                    else if (System.IO.File.Exists(filename))
                    {
                        ThreadPool.QueueUserWorkItem(loadProc, filename);
                        this.Invoke(new DPInt(ProgressAddMax), new object[] { progressBarLoad, 1 });
                    }
                }
            }
        }

        private bool processCUE(string filename)
        {
            StringBuilder str = new StringBuilder();
            str.AppendLine(filename);
            CD cd = CUEparse.openFile(filename);
            if (cd != null) {
                if (!cd.ToINSERTString().Equals("")) {
                    lock (SQLQue) { SQLQue.Enqueue(cd); }
                } else {
                    str.AppendLine("    CUEシートが無効です(おそらく実体のオーディオファイルが存在しません)");
                    trace(traceType.ERROR, str.ToString());
                    return false;
                }
            }
            trace(traceType.INFO, str.ToString());
            return true;
        }

        private void Form1_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            if ((importThread != null)&&(importThread.ThreadState != ThreadState.Stopped)) {
                return;
            }
            if (libraryDB == null) {
                OpenFileDialog openDLG = new OpenFileDialog();
                openDLG.Filter = "library.dbファイル|library.db";
                openDLG.DefaultExt = "DB";
                if (openDLG.ShowDialog() == DialogResult.OK) {
                    try {
                        libraryDB = new SQLite3DB(openDLG.FileName);
                    } catch (SQLite3DB.SQLite3Exception ex) {
                        trace(traceType.ERROR, ex.Message);
                    }
                }
            }
            importThread = new Thread(runImport);
            importThread.Start();
        }
    }
}


